==> react-app-logic-17.1.0.tgz: package/src/exchange/forms/useMarketOrderForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import {
  useAssetBalance,
  useDetailedBalance,
  useFixedAmountInput,
  useAsset,
} from '@onomy/react-hub';
import { OnexClient } from '@onomy/client';
import {
  useOnexPoolPrice,
  useFixedAmountFromDisplay,
  useOnexMarketEstimate,
  useOnexClient,
} from '../hooks';

type OrderDirection = 'buy' | 'sell';

export function useMarketOrderForm({
  baseDenom = 'anom',
  quoteDenom = 'usdt',
  orderDirection = 'buy' as OrderDirection,
}) {
  const onex = useOnexClient();
  const chain = onex?.getBlockchain() ?? null;
  const [direction, setDirection] = useState<OrderDirection>(orderDirection);

  const [baseAsset, baseAssetStatus] = useAsset(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset(quoteDenom, chain);
  const assetStatus = useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);

  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);

  const [txResult, setTxResult] = useState<null | Awaited<ReturnType<OnexClient['marketOrder']>>>(
    null
  );
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [hasAttemptedSubmit, setHasAttemptedSubmit] = useState(false);

  const [bidAmountFixed, setBidAmountFixed] = useState('0');
  const [slippagePercentage, setSlippagePercentage] = useState('3');

  const [estimatedAskAmount, estimateStatus] = useOnexMarketEstimate({
    denomAsk: direction === 'buy' ? baseDenom : quoteDenom,
    denomBid: direction === 'buy' ? quoteDenom : baseDenom,
    amountBid: bidAmountFixed,
  });

  const bidAsset = direction === 'buy' ? quoteAsset : baseAsset;
  const askAsset = direction === 'buy' ? baseAsset : quoteAsset;
  const bidBalance = direction === 'buy' ? quoteBalance : baseBalance;
  const askBalance = direction === 'buy' ? baseBalance : quoteBalance;

  const [bidAmount, bidAmountStatus] = useDetailedBalance(bidAsset, bidAmountFixed);
  const [askAmount, askAmountStatus] = useDetailedBalance(askAsset, estimatedAskAmount);
  const amountStatus = useAsyncStatuses([estimateStatus, bidAmountStatus, askAmountStatus]);

  const baseAmount = direction === 'buy' ? askAmount : bidAmount;
  const quoteAmount = direction === 'buy' ? bidAmount : askAmount;

  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    currentPriceFixed
  );

  const [bidAmountInput, setBidAmountInput] = useFixedAmountInput({
    value: bidAmountFixed,
    onChangeValue: setBidAmountFixed,
    meta: bidAsset.meta,
  });

  const estimatedPricePerBaseFixed = useMemo(() => {
    const bnEstAskFixed = new BigNumber(estimatedAskAmount);
    const bnBidAmountFixed = new BigNumber(bidAmountFixed);

    if (
      bnEstAskFixed.isNaN() ||
      bnEstAskFixed.eq(0) ||
      bnBidAmountFixed.isNaN() ||
      bnBidAmountFixed.eq(0)
    ) {
      return '0';
    }

    if (direction === 'buy') {
      const expShift = bidAsset.meta.decimalPlaces - askAsset.meta.decimalPlaces;
      const bnPrice = bnBidAmountFixed
        .dividedBy(bnEstAskFixed)
        .shiftedBy(expShift)
        .shiftedBy(bidAsset.meta.decimalPlaces);
      return bnPrice.toFixed(0);
    }
    const expShift = askAsset.meta.decimalPlaces - bidAsset.meta.decimalPlaces;
    const bnPrice = bnEstAskFixed
      .dividedBy(bnBidAmountFixed)
      .shiftedBy(expShift)
      .shiftedBy(askAsset.meta.decimalPlaces);
    return bnPrice.toFixed(0);
  }, [
    estimatedAskAmount,
    bidAmountFixed,
    bidAsset.meta.decimalPlaces,
    askAsset.meta.decimalPlaces,
    direction,
  ]);

  const [estimatedPricePerBase, estimatedPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    estimatedPricePerBaseFixed
  );

  const inputParams: Parameters<OnexClient['marketOrder']>[0] = useMemo(() => {
    const slippage = new BigNumber(slippagePercentage).times(100).toFixed(0);
    if (direction === 'buy') {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        amountBid: bidAmountFixed,
        amountAsk: estimatedAskAmount,
        slippage,
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      amountBid: bidAmountFixed,
      amountAsk: estimatedAskAmount,
      slippage,
    };
  }, [slippagePercentage, direction, baseDenom, quoteDenom, bidAmountFixed, estimatedAskAmount]);

  const validationErrors = useMemo(() => {
    const errors: {
      amountBid?: string;
      amountAsk?: string;
      slippage?: string;
      liquidity?: string;
    } = {};

    const bnAmountBid = new BigNumber(inputParams.amountBid);
    const bnAmountAsk = new BigNumber(inputParams.amountAsk);
    const bnSlippage = new BigNumber(inputParams.slippage);

    if (bnAmountBid.isNaN() || !bnAmountBid.isFinite() || bnAmountBid.lte(0)) {
      errors.amountBid = 'Must be a number > 0';
    }

    if (bnAmountAsk.isNaN() || !bnAmountAsk.isFinite() || bnAmountAsk.lte(0)) {
      errors.amountAsk = 'Must be a number > 0';
    }

    if (bnSlippage.isNaN() || !bnSlippage.isFinite() || bnSlippage.lte(0)) {
      errors.slippage = 'Must be a number > 0';
    }

    if (bnSlippage.gte(10000)) {
      errors.slippage = 'Must be a number < 100';
    }

    if (estimateStatus.error) {
      errors.liquidity = 'Insufficient liquidity in pool';
    }

    if (bnAmountBid.gt(bidBalance.amount.toString())) {
      errors.amountBid = 'Exceeds available balance';
    }

    /*
    if (bnAmountAsk.gt(askBalance.amount.toString())) {
      errors.amountAsk = 'Exceeds available balance (ask)';
    }
    */
    return Object.keys(errors).length > 0 ? errors : null;
  }, [inputParams, estimateStatus.error, bidBalance.amount]);

  const loadingStatus = useAsyncStatuses([assetStatus, balanceStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished && estimateStatus.finished;

  const submitTx = useCallback(
    (e?: React.FormEvent) => {
      e?.preventDefault();
      setHasAttemptedSubmit(true);
      if (!canSubmit) return Promise.resolve();
      resetTxStatus();
      setTxResult(null);
      return attemptTx(async () => setTxResult((await onex?.marketOrder(inputParams)) ?? null));
    },
    [canSubmit, inputParams, resetTxStatus, attemptTx, onex]
  );

  const reset = useCallback(() => {
    setHasAttemptedSubmit(false);
    resetTxStatus();
    setBidAmountFixed('0');
    setTxResult(null);
  }, [resetTxStatus]);

  useEffect(reset, [baseDenom, quoteDenom, direction, reset]);

  useEffect(() => {
    setDirection(orderDirection);
  }, [orderDirection]);

  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    estimateStatus,
    bidAmountStatus,
    askAmountStatus,
    amountStatus,
    currentPriceStatus,
    currentPricePerBaseStatus,
    estimatedPricePerAskStatus: estimatedPricePerBaseStatus,

    validationErrors,
    canSubmit,
    hasAttemptedSubmit,

    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    bidAsset,
    askAsset,
    bidAmount,
    askAmount,
    bidBalance,
    askBalance,
    baseAmount,
    quoteAmount,
    currentPricePerBase,
    estimatedPricePerBase,

    baseDenom,
    quoteDenom,
    direction,
    slippagePercentage,
    bidAmountInput,

    setDirection,
    setBidAmountInput,
    setSlippagePercentage,

    submitTx,
    reset,
  };
}

export type MarketOrderFormState = ReturnType<typeof useMarketOrderForm>;

==> react-app-logic-17.2.0.tgz: package/src/exchange/forms/useMarketOrderForm.ts <==
import { useCallback, useEffect, useMemo, useState } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncProcess, useAsyncStatuses } from '@onomy/react-utils';
import {
  useAssetBalance,
  useDetailedBalance,
  useFixedAmountInput,
  useAsset,
} from '@onomy/react-hub';
import { OnexClient } from '@onomy/client';
import {
  useOnexPoolPrice,
  useFixedAmountFromDisplay,
  useOnexMarketEstimate,
  useOnexClient,
} from '../hooks';

type OrderDirection = 'buy' | 'sell';

export function useMarketOrderForm({
  baseDenom = 'anom',
  quoteDenom = 'usdt',
  orderDirection = 'buy' as OrderDirection,
}) {
  const onex = useOnexClient();
  const chain = onex?.getBlockchain() ?? null;
  const [direction, setDirection] = useState<OrderDirection>(orderDirection);

  const [baseAsset, baseAssetStatus] = useAsset(baseDenom, chain);
  const [quoteAsset, quoteAssetStatus] = useAsset(quoteDenom, chain);
  const assetStatus = useAsyncStatuses([baseAssetStatus, quoteAssetStatus]);

  const [baseBalance, baseBalanceStatus] = useAssetBalance(baseAsset);
  const [quoteBalance, quoteBalanceStatus] = useAssetBalance(quoteAsset);
  const balanceStatus = useAsyncStatuses([baseBalanceStatus, quoteBalanceStatus]);

  const [txResult, setTxResult] = useState<null | Awaited<ReturnType<OnexClient['marketOrder']>>>(
    null
  );
  const [attemptTx, txStatus, resetTxStatus] = useAsyncProcess();
  const [hasAttemptedSubmit, setHasAttemptedSubmit] = useState(false);

  const [bidAmountFixed, setBidAmountFixed] = useState('0');
  const [slippagePercentage, setSlippagePercentage] = useState('3');

  const [estimatedAskAmount, estimateStatus] = useOnexMarketEstimate({
    denomAsk: direction === 'buy' ? baseDenom : quoteDenom,
    denomBid: direction === 'buy' ? quoteDenom : baseDenom,
    amountBid: bidAmountFixed,
  });

  const bidAsset = direction === 'buy' ? quoteAsset : baseAsset;
  const askAsset = direction === 'buy' ? baseAsset : quoteAsset;
  const bidBalance = direction === 'buy' ? quoteBalance : baseBalance;
  const askBalance = direction === 'buy' ? baseBalance : quoteBalance;

  const [bidAmount, bidAmountStatus] = useDetailedBalance(bidAsset, bidAmountFixed);
  const [askAmount, askAmountStatus] = useDetailedBalance(askAsset, estimatedAskAmount);
  const amountStatus = useAsyncStatuses([estimateStatus, bidAmountStatus, askAmountStatus]);

  const baseAmount = direction === 'buy' ? askAmount : bidAmount;
  const quoteAmount = direction === 'buy' ? bidAmount : askAmount;

  const [currentPrice, currentPriceStatus] = useOnexPoolPrice({ baseDenom, quoteDenom });
  const currentPriceFixed = useFixedAmountFromDisplay(currentPrice, quoteAsset);
  const [currentPricePerBase, currentPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    currentPriceFixed
  );

  const [bidAmountInput, setBidAmountInput] = useFixedAmountInput({
    value: bidAmountFixed,
    onChangeValue: setBidAmountFixed,
    meta: bidAsset.meta,
  });

  const estimatedPricePerBaseFixed = useMemo(() => {
    const bnEstAskFixed = new BigNumber(estimatedAskAmount);
    const bnBidAmountFixed = new BigNumber(bidAmountFixed);

    if (
      bnEstAskFixed.isNaN() ||
      bnEstAskFixed.eq(0) ||
      bnBidAmountFixed.isNaN() ||
      bnBidAmountFixed.eq(0)
    ) {
      return '0';
    }

    if (direction === 'buy') {
      const expShift = bidAsset.meta.decimalPlaces - askAsset.meta.decimalPlaces;
      const bnPrice = bnBidAmountFixed
        .dividedBy(bnEstAskFixed)
        .shiftedBy(expShift)
        .shiftedBy(bidAsset.meta.decimalPlaces);
      return bnPrice.toFixed(0);
    }
    const expShift = askAsset.meta.decimalPlaces - bidAsset.meta.decimalPlaces;
    const bnPrice = bnEstAskFixed
      .dividedBy(bnBidAmountFixed)
      .shiftedBy(expShift)
      .shiftedBy(askAsset.meta.decimalPlaces);
    return bnPrice.toFixed(0);
  }, [
    estimatedAskAmount,
    bidAmountFixed,
    bidAsset.meta.decimalPlaces,
    askAsset.meta.decimalPlaces,
    direction,
  ]);

  const [estimatedPricePerBase, estimatedPricePerBaseStatus] = useDetailedBalance(
    quoteAsset,
    estimatedPricePerBaseFixed
  );

  const inputParams: Parameters<OnexClient['marketOrder']>[0] = useMemo(() => {
    const slippage = new BigNumber(slippagePercentage).times(100).toFixed(0);
    if (direction === 'buy') {
      return {
        denomAsk: baseDenom,
        denomBid: quoteDenom,
        amountBid: bidAmountFixed,
        amountAsk: estimatedAskAmount,
        slippage,
      };
    }
    return {
      denomAsk: quoteDenom,
      denomBid: baseDenom,
      amountBid: bidAmountFixed,
      amountAsk: estimatedAskAmount,
      slippage,
    };
  }, [slippagePercentage, direction, baseDenom, quoteDenom, bidAmountFixed, estimatedAskAmount]);

  const validationErrors = useMemo(() => {
    const errors: {
      amountBid?: string;
      amountAsk?: string;
      slippage?: string;
      liquidity?: string;
    } = {};

    const bnAmountBid = new BigNumber(inputParams.amountBid);
    const bnAmountAsk = new BigNumber(inputParams.amountAsk);
    const bnSlippage = new BigNumber(inputParams.slippage);

    if (bnAmountBid.isNaN() || !bnAmountBid.isFinite() || bnAmountBid.lte(0)) {
      errors.amountBid = 'Must be a number > 0';
    }

    if (bnAmountAsk.isNaN() || !bnAmountAsk.isFinite() || bnAmountAsk.lte(0)) {
      errors.amountAsk = 'Must be a number > 0';
    }

    if (bnSlippage.isNaN() || !bnSlippage.isFinite() || bnSlippage.lt(0)) {
      errors.slippage = 'Must be a number >= 0';
    }

    if (bnSlippage.gte(10000)) {
      errors.slippage = 'Must be a number < 100';
    }

    if (estimateStatus.error) {
      errors.liquidity = 'Insufficient liquidity in pool';
    }

    if (bnAmountBid.gt(bidBalance.amount.toString())) {
      errors.amountBid = 'Exceeds available balance';
    }

    /*
    if (bnAmountAsk.gt(askBalance.amount.toString())) {
      errors.amountAsk = 'Exceeds available balance (ask)';
    }
    */
    return Object.keys(errors).length > 0 ? errors : null;
  }, [inputParams, estimateStatus.error, bidBalance.amount]);

  const loadingStatus = useAsyncStatuses([assetStatus, balanceStatus]);
  const canSubmit = !validationErrors && loadingStatus.finished && estimateStatus.finished;

  const submitTx = useCallback(
    (e?: React.FormEvent) => {
      e?.preventDefault();
      setHasAttemptedSubmit(true);
      if (!canSubmit) return Promise.resolve();
      resetTxStatus();
      setTxResult(null);
      return attemptTx(async () => setTxResult((await onex?.marketOrder(inputParams)) ?? null));
    },
    [canSubmit, inputParams, resetTxStatus, attemptTx, onex]
  );

  const reset = useCallback(() => {
    setHasAttemptedSubmit(false);
    resetTxStatus();
    setBidAmountFixed('0');
    setTxResult(null);
  }, [resetTxStatus]);

  useEffect(reset, [baseDenom, quoteDenom, direction, reset]);

  useEffect(() => {
    setDirection(orderDirection);
  }, [orderDirection]);

  return {
    txStatus,
    txResult,
    assetStatus,
    balanceStatus,
    loadingStatus,
    estimateStatus,
    bidAmountStatus,
    askAmountStatus,
    amountStatus,
    currentPriceStatus,
    currentPricePerBaseStatus,
    estimatedPricePerAskStatus: estimatedPricePerBaseStatus,

    validationErrors,
    canSubmit,
    hasAttemptedSubmit,

    baseAsset,
    baseBalance,
    quoteAsset,
    quoteBalance,
    bidAsset,
    askAsset,
    bidAmount,
    askAmount,
    bidBalance,
    askBalance,
    baseAmount,
    quoteAmount,
    currentPricePerBase,
    estimatedPricePerBase,

    baseDenom,
    quoteDenom,
    direction,
    slippagePercentage,
    bidAmountInput,

    setDirection,
    setBidAmountInput,
    setSlippagePercentage,

    submitTx,
    reset,
  };
}

export type MarketOrderFormState = ReturnType<typeof useMarketOrderForm>;
